@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2007-2011 The HyperSpy developers
+# Copyright 2007-2021 The HyperSpy developers
 #
 # This file is part of  HyperSpy.
 #
@@ -16,20 +16,24 @@
 # You should have received a copy of the GNU General Public License
 # along with  HyperSpy.  If not, see <http://www.gnu.org/licenses/>.
 
-from __future__ import division
+
 import math
+import logging
 
 import numpy as np
 from scipy.interpolate import splev
 
-from hyperspy.defaults_parser import preferences
 from hyperspy.component import Component
-from hyperspy import messages
 from hyperspy.misc.eels.hartree_slater_gos import HartreeSlaterGOS
 from hyperspy.misc.eels.hydrogenic_gos import HydrogenicGOS
 from hyperspy.misc.eels.effective_angle import effective_angle
+from hyperspy.ui_registry import add_gui_method
 
 
+_logger = logging.getLogger(__name__)
+
+
+@add_gui_method(toolkey="hyperspy.EELSCLEdge_Component")
 class EELSCLEdge(Component):
 
     """EELS core loss ionisation edge from hydrogenic or tabulated
@@ -38,21 +42,19 @@ class EELSCLEdge(Component):
     Hydrogenic GOS are limited to K and L shells.
 
     Currently it only supports Peter Rez's Hartree Slater cross sections
-    parametrised as distributed by Gatan in their
-    Digital Micrograph (DM) software. If Digital Micrograph is installed
-    in the system HyperSpy in the standard location HyperSpy should
-    find the path to the HS GOS folder. Otherwise, the location of the
-    folder can be defined in HyperSpy preferences, which can be done through
-        preferences.gui() or the preferences.EELS.eels_gos_files_path variable.
-
-    Calling this class with a numpy.array
-
+    parametrised as distributed by Gatan in their Digital Micrograph (DM)
+    software. If Digital Micrograph is installed in the system HyperSpy in the
+    standard location HyperSpy should find the path to the HS GOS folder.
+    Otherwise, the location of the folder can be defined in HyperSpy
+    preferences, which can be done through hs.preferences.gui() or the
+    hs.preferences.EELS.eels_gos_files_path variable.
 
     Parameters
     ----------
-    element_subshell : str
-            For example, 'Ti_L3' for the GOS of the titanium L3 subshell
-
+    element_subshell : {str, dict}
+        Usually a string, for example, 'Ti_L3' for the GOS of the titanium L3
+        subshell. If a dictionary is passed, it is assumed that Hartree Slater
+        GOS was exported using `GOS.as_dictionary`, and will be reconstructed.
     GOS : {'hydrogenic', 'Hartree-Slater', None}
         The GOS to use. If None it will use the Hartree-Slater GOS if
         they are available, otherwise it will use the hydrogenic GOS.
@@ -71,20 +73,17 @@ class EELSCLEdge(Component):
         Fix this parameter to fix the fine structure. It is a
         component.Parameter instance.
     effective_angle : Parameter
-        The effective collection angle. It is automatically
+        The effective collection semi-angle. It is automatically
         calculated by set_microscope_parameters. It is a
         component.Parameter instance. It is fixed by default.
     fine_structure_smoothing : float between 0 and 1
         Controls the level of smoothing of the fine structure model.
         Decreasing the value increases the level of smoothing.
-
     fine_structure_active : bool
-        Activates/deactivates the fine structure feature. Its
-        default value can be choosen in the preferences.
+        Activates/deactivates the fine structure feature.
 
     """
-    _fine_structure_smoothing = \
-        preferences.EELS.fine_structure_smoothing
+    _fine_structure_smoothing = 0.3
 
     def __init__(self, element_subshell, GOS=None):
         # Declare the parameters
@@ -93,12 +92,16 @@ class EELSCLEdge(Component):
                             'fine_structure_coeff',
                             'effective_angle',
                             'onset_energy'])
-        self.name = element_subshell
-        self.element, self.subshell = element_subshell.split('_')
+        if isinstance(element_subshell, dict):
+            self.element = element_subshell['element']
+            self.subshell = element_subshell['subshell']
+        else:
+            self.element, self.subshell = element_subshell.split('_')
+        self.name = "_".join([self.element, self.subshell])
         self.energy_scale = None
         self.effective_angle.free = False
-        self.fine_structure_active = preferences.EELS.fine_structure_active
-        self.fine_structure_width = preferences.EELS.fine_structure_width
+        self.fine_structure_active = False
+        self.fine_structure_width = 30.
         self.fine_structure_coeff.ext_force_positive = False
         self.GOS = None
         # Set initial actions
@@ -108,8 +111,8 @@ class EELSCLEdge(Component):
                 GOS = 'Hartree-Slater'
             except IOError:
                 GOS = 'hydrogenic'
-                messages.information(
-                    'Hartree-Slater GOS not available'
+                _logger.info(
+                    'Hartree-Slater GOS not available. '
                     'Using hydrogenic GOS')
         if self.GOS is None:
             if GOS == 'Hartree-Slater':
@@ -129,6 +132,22 @@ class EELSCLEdge(Component):
         self.intensity.bmin = 0.
         self.intensity.bmax = None
 
+        self._whitelist['GOS'] = ('init', GOS)
+        if GOS == 'Hartree-Slater':
+            self._whitelist['element_subshell'] = (
+                'init',
+                self.GOS.as_dictionary(True))
+        elif GOS == 'hydrogenic':
+            self._whitelist['element_subshell'] = ('init', element_subshell)
+        self._whitelist['fine_structure_active'] = None
+        self._whitelist['fine_structure_width'] = None
+        self._whitelist['fine_structure_smoothing'] = None
+        self.effective_angle.events.value_changed.connect(
+            self._integrate_GOS, [])
+        self.onset_energy.events.value_changed.connect(self._integrate_GOS, [])
+        self.onset_energy.events.value_changed.connect(
+            self._calculate_knots, [])
+
     # Automatically fix the fine structure when the fine structure is
     # disable.
     # In this way we avoid a common source of problems when fitting
@@ -164,7 +183,7 @@ class EELSCLEdge(Component):
         self._calculate_effective_angle()
     E0 = property(_get_E0, _set_E0)
 
-    # Collection angles
+    # Collection semi-angle
     def _get_collection_angle(self):
         return self.__collection_angle
 
@@ -173,7 +192,7 @@ class EELSCLEdge(Component):
         self._calculate_effective_angle()
     collection_angle = property(_get_collection_angle,
                                 _set_collection_angle)
-    # Convergence angle
+    # Convergence semi-angle
 
     def _get_convergence_angle(self):
         return self.__convergence_angle
@@ -191,18 +210,12 @@ class EELSCLEdge(Component):
                 self.GOS.onset_energy,
                 self.convergence_angle,
                 self.collection_angle)
-        except:
+        except BaseException:
             # All the parameters may not be defined yet...
             pass
 
     @property
     def fine_structure_smoothing(self):
-        """Controls the level of the smoothing of the fine structure.
-
-        It must a real number between 0 and 1. The higher close to 0
-        the higher the smoothing.
-
-        """
         return self._fine_structure_smoothing
 
     @fine_structure_smoothing.setter
@@ -238,19 +251,22 @@ class EELSCLEdge(Component):
         E0 : float
             Electron beam energy in keV.
         alpha: float
-            Convergence angle in mrad.
+            Convergence semi-angle in mrad.
         beta: float
-            Collection angle in mrad.
+            Collection semi-angle in mrad.
         energy_scale : float
             The energy step in eV.
         """
         # Relativistic correction factors
-
-        self.convergence_angle = alpha
-        self.collection_angle = beta
-        self.energy_scale = energy_scale
-        self.E0 = E0
-        self._integrate_GOS()
+        old = self.effective_angle.value
+        with self.effective_angle.events.value_changed.suppress_callback(
+                self._integrate_GOS):
+            self.convergence_angle = alpha
+            self.collection_angle = beta
+            self.energy_scale = energy_scale
+            self.E0 = E0
+        if self.effective_angle.value != old:
+            self._integrate_GOS()
 
     def _integrate_GOS(self):
         # Integration over q using splines
@@ -262,42 +278,50 @@ class EELSCLEdge(Component):
         E2 = self.GOS.energy_axis[-1] + self.GOS.energy_shift
         y1 = self.GOS.qint[-2]  # in m**2/bin */
         y2 = self.GOS.qint[-1]  # in m**2/bin */
-        self.r = math.log(y2 / y1) / math.log(E1 / E2)
-        self.A = y1 / E1 ** -self.r
-
-        # Connect them at this point where it is certain that all the
-        # parameters are well defined
-        self.effective_angle.connect(self._integrate_GOS)
-        self.onset_energy.connect(self._integrate_GOS)
-        self.onset_energy.connect(self._calculate_knots)
+        self._power_law_r = math.log(y2 / y1) / math.log(E1 / E2)
+        self._power_law_A = y1 / E1 ** -self._power_law_r
 
     def _calculate_knots(self):
         start = self.onset_energy.value
         stop = start + self.fine_structure_width
-        self.__knots = np.r_[[start] * 4,
-                             np.linspace(start,
-                                         stop,
-                                         self.fine_structure_coeff._number_of_elements
-                                         )[2:-2], [stop] * 4]
+        self.__knots = np.r_[
+            [start] * 4,
+            np.linspace(
+                start,
+                stop,
+                self.fine_structure_coeff._number_of_elements)[
+                2:-2],
+            [stop] * 4]
 
     def function(self, E):
         """Returns the number of counts in barns
 
         """
+        shift = self.onset_energy.value - self.GOS.onset_energy
+        if shift != self.GOS.energy_shift:
+            # Because hspy Events are not executed in any given order,
+            # an external function could be in the same event execution list
+            # as _integrate_GOS and be executed first. That can potentially
+            # cause an error that enforcing _integrate_GOS here prevents. Note
+            # that this is suboptimal because _integrate_GOS is computed twice
+            # unnecessarily.
+            self._integrate_GOS()
         Emax = self.GOS.energy_axis[-1] + self.GOS.energy_shift
         cts = np.zeros((len(E)))
         bsignal = (E >= self.onset_energy.value)
         if self.fine_structure_active is True:
             bfs = bsignal * (
                 E < (self.onset_energy.value + self.fine_structure_width))
-            cts[bfs] = splev(E[bfs],
-                             (self.__knots, self.fine_structure_coeff.value + (0,) * 4,
-                              3))
+            cts[bfs] = splev(
+                E[bfs], (
+                    self.__knots,
+                    self.fine_structure_coeff.value + (0,) * 4,
+                    3))
             bsignal[bfs] = False
         itab = bsignal * (E <= Emax)
         cts[itab] = self.tab_xsection(E[itab])
         bsignal[itab] = False
-        cts[bsignal] = self.A * E[bsignal] ** -self.r
+        cts[bsignal] = self._power_law_A * E[bsignal] ** -self._power_law_r
         return cts * self.intensity.value
 
     def grad_intensity(self, E):
@@ -313,10 +337,11 @@ class EELSCLEdge(Component):
         if len(fs) == len(self.__knots):
             self.fine_structure_coeff.value = fs
         else:
-            messages.warning_exit("The provided fine structure file "
-                                  "doesn't match the size of the current fine structure")
+            raise ValueError(
+                "The provided fine structure file "
+                "doesn't match the size of the current fine structure")
 
-    def get_fine_structure_as_spectrum(self):
+    def get_fine_structure_as_signal1D(self):
         """Returns a spectrum containing the fine structure.
 
         Notes
