@@ -24,8 +24,10 @@ from hyperspy import components1d
 from hyperspy._signals.eels import EELSSpectrum
 from hyperspy.components1d import EELSCLEdge, PowerLaw
 from hyperspy.docstrings.model import FIT_PARAMETERS_ARG
+from hyperspy.misc.utils import dummy_context_manager
 from hyperspy.models.model1d import Model1D
 
+
 _logger = logging.getLogger(__name__)
 
 
@@ -350,12 +352,14 @@ class EELSModel(Model1D):
         * :py:meth:`~hyperspy.model.EELSModel.fit`
 
         """
-        # Fit background
-        self.fit_background(start_energy, **kwargs)
+        cm = self.suspend_update if self._plot_active else dummy_context_manager
+        with cm(update_on_resume=True):
+            # Fit background
+            self.fit_background(start_energy, **kwargs)
 
-        # Fit the edges
-        for i in range(0, len(self._active_edges)):
-            self._fit_edge(i, start_energy, **kwargs)
+            # Fit the edges
+            for i in range(0, len(self._active_edges)):
+                self._fit_edge(i, start_energy, **kwargs)
 
     smart_fit.__doc__ %= FIT_PARAMETERS_ARG
 
@@ -734,11 +738,29 @@ class EELSModel(Model1D):
         self.resolve_fine_structure()
 
     def set_all_edges_intensities_positive(self):
+        """
+        Set all edges intensities positive by setting ``ext_force_positive``
+        and ``ext_bounded`` to ``True``.
+
+        Returns
+        -------
+        None.
+
+        """
         for edge in self._active_edges:
             edge.intensity.ext_force_positive = True
             edge.intensity.ext_bounded = True
 
     def unset_all_edges_intensities_positive(self):
+        """
+        Unset all edges intensities positive by setting ``ext_force_positive``
+        and ``ext_bounded`` to ``False``.
+
+        Returns
+        -------
+        None.
+
+        """
         for edge in self._active_edges:
             edge.intensity.ext_force_positive = False
             edge.intensity.ext_bounded = False
